#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import SLiCAP as sl
import sympy as sp

sl.initProject('Circuit T1 matrix solutions')

files = ['cir-1']

# In all circuit files, use:
    
# Signal source: V1
# Load resistor: R1; model = 'r', value = 'R_ell'
# The name of the input node of the circuit: i'
# The name of the output node of the circuit: 'o'

# Define the symbolic variables

# The load resistance:
    
R_s, R_ell = sp.symbols('R_s, R_ell')

# The input vector:

ViIi = sp.Matrix([[sp.Symbol('V_i')], [sp.Symbol('I_i')]])

# The output vector:

VoIo = sp.Matrix([[sp.Symbol('V_o')], [sp.Symbol('I_o')]])

# Calculate and display the T1 matrix equation of the circuit(s):

for fileName in files:
    kicadFile = 'kicad/' + fileName +'/' + fileName +'.kicad_sch'
    cir = sl.makeCircuit(kicadFile)
    
    V_i =  sl.doLaplace(cir, source='V1', detector='V_i').laplace
    I_i = -sl.doLaplace(cir, source='V1', detector='I_V1').laplace
    V_o =  sl.doLaplace(cir, source='V1', detector='V_o').laplace
    I_o =  sl.doLaplace(cir, source='V1', detector='I_R1').laplace
    
    # Calculate the T1 parameters
    
    A = sp.simplify(sp.limit(V_i/V_o, R_ell, 'oo'))
    B = sp.simplify(sp.limit(V_i/I_o, R_ell, 0))
    C = sp.simplify(sp.limit(I_i/V_o, R_ell, 'oo'))
    D = sp.simplify(sp.limit(I_i/I_o, R_ell, 0))
    
    # Create the T1 matrix
    
    T = sp.Matrix([[A, B], [C, D]])
    
    # Store the matrices in a result object for displaying on an HTML page
    
    result    = sl.doMatrix(cir)
    result.M  = T
    result.Iv = ViIi
    result.Dv = VoIo
    
    # Calculate all six transfers
    
    Z_i = sp.simplify((A*R_ell+B)/(C*R_ell+D))         # Input impedance
    Z_o = sp.simplify((D*R_s+B)/(C*R_s+A))             # Output impedance
    A_v = sp.simplify(1/(A+B/R_ell+C*R_s+D*R_s/R_ell)) # Voltage gain
    A_y = sp.simplify(1/(A*R_ell+B+C*R_s*R_ell+D*R_s)) # Transadmittance
    A_z = sp.simplify(1/(A/R_s+B/R_ell/R_s+C+D/R_ell)) # Transimpedance
    A_i = sp.simplify(1/(A*R_ell/R_s+B/R_s+C*R_ell+D)) # Current gain
    
    # HTML Report #############################################################
    
    sl.htmlPage('Determination of T1 matrix parameters')
    sl.head2html('Circuit diagram')
    sl.img2html(fileName + '.svg', 500)
    sl.head2html('T1 matrix ')
    sl.text2html('The T1 matrix of the device under test is found as:')
    sl.eqn2html('T', T)
    sl.text2html('The matrix equation for the two-port (DUT) is found as:')
    sl.matrices2html(result)
    sl.head2html("Input and output impedances")
    sl.eqn2html("Z_i", Z_i)
    sl.eqn2html("Z_o", Z_o)
    sl.head2html("Transfers")
    sl.eqn2html("A_v", A_v)
    sl.eqn2html("A_y", A_y)
    sl.text2html("For $A_z$ and $A_i$, the voltage source V1 in series with " +
                 "R4 is replaced with a current source in parallel with R4.")
    sl.eqn2html("A_z", A_z)
    sl.eqn2html("A_i", A_i)